import java.math.BigInteger;

import AlZAGK.MatrixBigInt.MatrixBigInt;
import AlZAGK.VektorBigInt.VektorBigInt;


/**
 * Diese Klasse verwaltet die RList-Objekte und enthält alle Methoden.
 * @author Nils Przigoda
 * 
 */
class Liste {
	
	VektorBigInt b;
	VektorBigInt c;
	MatrixBigInt A;
	RList Knoten;
	
	/**
	 * RList - Konstruktor erzeugt eine neue leere Liste.	 
	 */	
	public Liste(VektorBigInt b1, VektorBigInt c1, MatrixBigInt A1) {
		b = b1;
		c = c1;
		A = A1;
		Knoten = new RList(); 
	}
	
	
	private static void o(Object o) {
		//System.out.println(o);
	}
	
	/**
	 * Gibt an ob der Vektor x <c y ist
	 * @param x erster Vektor 
	 * @param y zweiter Vektor
	 * @param c Vergleichsvektor
	 * @return true wenn kleiner
	 * @return false wenn größergleich
	 */
	public boolean toc  (VektorBigInt x, VektorBigInt y, VektorBigInt c) {
    	if (x.length != y.length) {
    		System.out.println("Vektoren verschieden lang");
    		return false;
    	}
    	if (x.term_order_c(c).compareTo(y.term_order_c(c)) < 0) {
    		return true;
    	} else if (x.term_order_c(c) == y.term_order_c(c)) 
    		if (x.smaleq(y)) 
    			return true;    	    	
    	return false;
    }
	
	/**
	 * rlIsEmpty - Überprüft, ob die Liste leer ist und liefert true oder 
	 * false zurück
	 */
	public boolean rlIsEmpty() {
		return this.Knoten.data.equals(this.Knoten.n.data);
	}
	
	/**
	 * rlLen - Liefer die Länge einer Liste als Integer zurück, unabhängig 
	 * davon ob der Anker vorliegt oder nicht.  
	 */
	public int rlLen() {
		int counter=0;
		RList tmp=Knoten;
		while (!tmp.n.equals(Knoten)) {
			tmp = tmp.n;
			counter++;
		}
		return counter;
	} 		
	
	/**
	 * dient dem einfachen einfügen eines Objekts direkt hinter dem angebenem Knoten
	 * @param inhalt
	 * @param l
	 */
	private void rlApp(VektorBigInt inhalt, RList l) {
		RList tmp = new RList(inhalt);
		tmp.n = l.n;
		tmp.p = l;
		l.n.p = tmp;
		l.n = tmp;
	}
	
	/**
	 * rlReset - Löscht den Listeninhalt.
	 */
	public void rlReset() {
		if (Knoten.data == null) {
			Knoten.n = Knoten;
			Knoten.p =Knoten;
		}	else {
			System.out.println("Es wurde nicht der Anker übergeben");			
		}				
	}
	
	/**
	 * rlRead - Gibt den Inhalt des Knotes aus.
	 */
	public Object rlRead() {
    	if (Knoten == null) { // l undefiniert
    		return "";
    	}
    	if (Knoten.data==null) { // l ist Ankerelement
    		return null;
    	} 
    	// entweder wurde die Funktion abgebrochen oder es wird jetzt l.data returned
    	return Knoten.data;    	    
    }
	
	/**
	 * rlHead() gibt das erste Elemente der sortierten Liste zurück,
	 * also das direkt hinter dem Ankerelement
	 * @return gibt den Head der Liste wieder
	 */
	public Object rlHead() {
    	if (Knoten == null) { // l undefiniert
    		return null;
    	}
    	if (rlLen() == 0) { // leere Liste
    		return null;
    	}
    	/* Im optimalen Fall ist der Knoten bereits der Anker,
    	 * daher nehmen wir das vorherige Element, damit
    	 * die do-while möglichst schnell abbricht.
    	 */
    	RList tmp = Knoten.p;
    	// bis zum Ankerelement vorlaufen
    	do {
			tmp = tmp.n;
		} while (tmp.data != null);
    	return tmp.n.data;    	        
    }
	
	/**
	 * rlNext - Gibt den Nachfolger zurück.
	 */
	public void rlNext() {		
	    	// if falls undefiniert
	    	if (Knoten == null) {
	    		System.out.println("Unerwarteter Fehler Null-Ref.");
	    		return;
	    	} else {    		
	    		Knoten=Knoten.n;    		
	    	}
	}
	
	/**
	 * rlPrev - Gibt den Vorgänger zurück.
	 */
	public void rlPrev() {
//		 if falls undefiniert
    	if (Knoten == null) {
    		System.out.println("Unerwarteter Fehler Null-Ref.");
    		return;
    	} else {    		
    		Knoten=Knoten.p;    		
    	}
	}
	
	/**
	 * rlFind - Gibt die Referenz auf ein Objekt mit dem gesuchten String 
	 * zurück ansonsten wird "null" zurückgegeben.
	 */
	public RList rlFind(VektorBigInt finde) {
		if (finde == null)
			o("NULL");		
		RList tmp=Knoten.n;    	
		while ( Knoten.data != tmp.data ) {
			if (tmp.data != null)
				if (tmp.data.equals(finde)) 
					return tmp;
    		tmp=tmp.n;    		
    	}     	
		return null;		
	}
	
    /**
     * Methode rlPrint() gibt den Nutzdateninhalt und die
     * Länge der Liste aus.
     * 
     */
	public void rlPrint() {		
    	RList tmp = Knoten;
    	do {
    		tmp=tmp.n;    		
    	} while (tmp.data != null);
    	tmp=tmp.n;
    	System.out.println("Die Länge der folgdenen Liste beträgt: " + this.rlLen());    	
    	do {
    		if (tmp.data != null) {
    		 tmp.data.printZ();
    		 System.out.println("termc: " +
    		 	tmp.data.term_order_c(c));
    		}
    		tmp=tmp.n;
    	} while (tmp.data !=null);    	    	
    }	
	
	
    /**
     * Methode rlDel() löscht das aktuelle referenzierte 
     * Listenelement. Wirkungslos bei Referenz auf
     * Ankerelement.
     *
     *  @note Diese Operation benötigt eine if-Abfrage.
     *        Warum ?
     */
    public void rlDel(RList l) {
    	/*
    	 * Wenn l Anker ist, wird nix geändert,
    	 * ansonsten würde der Anker gelöscht werden, 
    	 * wenn mehr Elemente als nur der Anker in 
    	 * der Liste wären
    	 */
    	if (l.data != null) {
    		
    		l.p.n=l.n;
    		l.n.p=l.p;    		
    	}    	
    }
    
    
    /**
     * rlInsSorted - Fügt ein neues Objekt sortiert ein.
     * @param inhalt Der Inhalt des neuen Objekts
     */ 
    public void rlInsSorted(VektorBigInt inhalt) {
    	o("-=INSERT BEGINN=-");
    	o("-=Inhalt Vektor BEGINN=-");
    	//inhalt.printZ();    	
    	BigInteger inhalt_toc = inhalt.term_order_c(c);
    	o("toc= "+ inhalt_toc); 
    	
    	o("-=Inhalt Vektor  ENDE =-");    	
    	for (int i=0; i < inhalt.length;i++) {
			if (inhalt.vektor[i].compareTo(new BigInteger("0")) < 0) {
				o("Vektor  < 0 ");
				return;						
			}
		}
    	// Gucken ob der Vektor schon enthalten ist
    	RList t = rlFind(inhalt); 
    	if (t == null)
    		;
    	else {
    		o("Vektor bereits vorhanden");
			return;
    	}
    	RList tmp = Knoten;
    	if (tmp.data != null) {
    		o("TMP ist ungleich null");
    		do {
    			tmp = tmp.n;
    		} while (tmp.data != null);
    	} else
    		o("TMP ist gleich null");
    	if (tmp.data != null)
    		o("TMP ist immernoch ungleich null");
    	o("-=aktuelle Liste BEGINN=-");
    	//rlPrint();    	
    	o("-=aktuelle Liste  ENDE =-");
    	boolean weiter = false;
    	do {
			tmp = tmp.n;
			if (tmp.data != null) {
				weiter = toc(tmp.data,inhalt,c);
				//tmp.data.printZ();
				o("weiter= "+ weiter+ " || tmp toc=" +
						tmp.data.term_order_c(c) + " || " +
						"inhalt toc " + inhalt_toc);
				o("while bed: " + (tmp.data != null && weiter));
			}
		} while (tmp.data != null && weiter);
    	//if (tmp.data != null)
    	//	tmp.data.printZ();
    	// ggf. anpassen für Vektoren oder ähnlich
		VektorBigInt value = A.MultSp(inhalt);
		//inhalt.printZ();
		o("VALUE ist gleich: ");				
		//value.printZ();		
		if (b.equals(value))
			rlApp(inhalt,tmp.p);			
		else
			o("Nicht zulässiger Wert");
		o("-=INSERT ENDE=-");
    }
	
	/**
	 * rlCat fügt 2 Liste zusammen 
	 * @param l ist die einzufügende Liste
	 *  
	 */
    public void rlCat(RList l) {
    	Knoten.p.n = l.n;    	
    	l.p.n = Knoten;    	
    	l.n.p = Knoten.p;
    	Knoten.p = l.p;
    	/*
    	 * Die folgeden zwei Zeilen eigentlich
    	 * überflüßig nach Aufgabenstellung, 
    	 * jedoch sollte man noch die 2. Liste
    	 * noch nur aus dem Ankerelemen bestehen,
    	 * damit sie ggf noch ausgegeben werden kann.
    	 */
    	l.p=l;
    	l.n=l;
    }
    
	/**
	 * rlEquals - vergleicht 2 Listen miteinander und gibt 	
	 */
	public boolean rlEquals(Liste l) {		
    	if (this.rlLen() != l.rlLen()) {
    		return false;
    	}
    	Liste temp = this;
    	do {
    		if (temp.Knoten.data != l.Knoten.data) {
    			return false;
    		}
    		temp.Knoten=temp.Knoten.n;
    		l.Knoten=l.Knoten.n;
    	} while (temp != temp);
    	return true;
	}	
}
