package AlZAGK.MatrixBigInt;

import AlZAGK.VektorBigInt.VektorBigInt;

import java.util.Arrays;
import java.math.BigInteger;

public class MatrixBigInt {	
    private final BigInteger[][] matrix;
    public final int Spalten;
    public final int Zeilen;

    public MatrixBigInt() {
    	matrix = new BigInteger[0][0];
    	Spalten = -1;
    	Zeilen = -1;
    }
    
    public MatrixBigInt(BigInteger[][] m) {
    	matrix = m;
    	// o(m.length + " | " + m[0].length);
    	Spalten = m[0].length;
    	// Erster Zugriff ist immer auf die Spalte/n!
    	Zeilen = m.length;    	
    }
    
    public BigInteger Give_Element (int Sp, int Z) {
    	if (Spalten == -1 || Zeilen == -1) {
    		o("Was für eine Matrix?");
    		return null;
    	}
    	// Sp > Spalten und analog zu Zeilen abfangen 
    	return this.matrix[Z][Sp];
    }
    
    private static void o(Object o) {
		System.out.println(o);
	}
    
    /**
     * Erzeugt eine n x n-Dim. Einheits-Matrix
     * @param n Anzahl der Zeilen und Spalten
     */
    public MatrixBigInt(int n) {
    	matrix = new BigInteger[n][n];    	
    	Spalten = n;    	
    	Zeilen = n;
    	for (int i=0; i < this.Spalten || i < this.Zeilen; i++) {        		
    		Arrays.fill(matrix[i], new BigInteger("0"));
    		matrix[i][i] = new BigInteger("1");
    	}
    }
    
    /**
     * Erzeugt eine m x n-Dim. Matrix mit 1en auf der Hauptdiagonalen
     * @param Sp Spalten = m 
     * @param Z  Zeilen = n
     */
    public MatrixBigInt(int Sp, int Z) {
    	matrix = new BigInteger[Sp][Z];    	
    	Spalten = Sp;    	
    	Zeilen = Z;
    	for (int i=0; i < this.Spalten || i < this.Zeilen; i++) {        		
    		Arrays.fill(matrix[i], new BigInteger("0"));
    		matrix[i][i] = new BigInteger("1");
    	}    	     
    }        
    
    /**
     * Vergleicht die übergebene Matrix mit der akt. 
     * @param a Matrix mit der verglichen werden soll
     * @return true falls Matrizen gleich sind
     * @return false wenn die Spalten oder Zeilen nicht übereinstimmen
     * @return false wenn sich ein Eintrag unterscheidet 
     */
    public boolean equals(MatrixBigInt a) {
    	if (a.Spalten != this.Spalten
    			|| a.Zeilen != this.Zeilen)
    		return false;
    	for (int i=0; i < this.Spalten;i++) 
    		for (int j=0; j < this.Spalten;j++) 
        		if (!a.matrix[i][j].equals(this.matrix[i][j]))
        			return false;        		        	
        return true;        
    } 
    
    public MatrixBigInt Add(MatrixBigInt m) {
    	if (this.Spalten != m.Spalten
    			&& this.Zeilen != m.Zeilen) {
    		System.out.println("Matrizen haben unterschiedliche" +
    				"Größen");
    		return (new MatrixBigInt());
    	}
    	BigInteger[][] temp = new BigInteger[Spalten][Zeilen];
    	MatrixBigInt tmp = new MatrixBigInt (temp);    	
    	for (int i=0; i < this.Spalten ; i++) 
    		for (int j=0; j < this.Zeilen ; j++) 
    		tmp.matrix[i][j] = matrix[i][j].add(m.matrix[i][j]);    	    	
    	return tmp;
    }
    
    public MatrixBigInt Sub(MatrixBigInt m) {
    	if (this.Spalten != m.Spalten
    			&& this.Zeilen != m.Zeilen) {
    		System.out.println("Matrizen haben unterschiedliche" +
    				"Größen");
    		return (new MatrixBigInt());
    	}
    	BigInteger[][] temp = new BigInteger[Spalten][Zeilen];
    	MatrixBigInt tmp = new MatrixBigInt (temp);    	
    	for (int i=0; i < this.Spalten ; i++) 
    		for (int j=0; j < this.Zeilen ; j++) 
    		tmp.matrix[i][j] = matrix[i][j].subtract(m.matrix[i][j]);    	    	
    	return tmp;
    }  
        
    public VektorBigInt MultSp (VektorBigInt v) {
    	if (Spalten != v.length) {
    		System.out.println("Matrix-Spalten: " + Spalten + 
    				" || Vektorlaenge: " + v.length);

    		return new VektorBigInt();
    	}    	
    	VektorBigInt tmp = new VektorBigInt (new BigInteger[Zeilen]);
    	BigInteger sum = new BigInteger("0");
    	for (int i=0; i < this.Zeilen ; i++) {
    		sum = new BigInteger("0");
    		for (int j=0; j < this.Spalten ; j++) 
    			sum = sum.add( (matrix[i][j].multiply(v.vektor[j])) );
    		tmp.vektor[i] = sum;
    	}
    	return tmp; 
    }
    
    public VektorBigInt MultZ (VektorBigInt v) {
    	if (this.Zeilen != v.length) {
    		System.out.println("Vektoren-Matrixlänge nicht richtig");
    		return new VektorBigInt();
    	}
    	BigInteger[] temp = new BigInteger[Zeilen];
    	VektorBigInt tmp = new VektorBigInt (temp);
    	BigInteger sum = new BigInteger("0");
    	for (int i=0; i < this.Zeilen ; i++) {
    		sum = new BigInteger("0");
    		for (int j=0; j < this.Spalten ; j++) 
    			sum = sum.add(matrix[i][j].multiply(v.vektor[j]));    			     			                
    		tmp.vektor[i] = sum;
    	}
    	return tmp; 
    }
    
    public void print () {
    	System.out.println(this.Zeilen + " x " + this.Spalten);
    	for (int i=0; i < this.Zeilen ; i++) {
    		for (int j=0; j < this.Spalten ; j++) 
    			System.out.print(" " + matrix[i][j]);
    		System.out.print("\n");
    	}
    	System.out.println();
    }
	
}