/*
 * Matlab Kontrolle mit:
with(PolynomialIdeals):
J := <y_1-x_1, y_1*y_2-x_2, y_1*y_2^2-x_3, y_1*y_2^3-x_4>:
GroebnerBasis(J,wdeg([100000000, 100000000, 1, 3, 14, 17],[y_1,y_2,x_1,x_2,x_3,x_4])); 
 */
package AlZAGK.numbers.Polynom;

import java.io.IOException;
import java.math.BigInteger;
import java.util.Arrays;

import AlZAGK.VektorBigInt.VektorBigInt;
import AlZAGK.numbers.Rational.Rational;

/**
 * Hilfsobjekt zur Verwaltung der Paare in der Menge
 * P bei der Berechnung der Gröbner-Basis
 * @author Nils Przigoda
 * @version 0.1
 *
 */
class J_Paar {
	Polynom poly;
	Polynom monom;
		
	J_Paar(Polynom pol, Polynom mon) {
		poly = pol; 
		monom = mon;	
	}	
}

/**
 * Hilfsobjekt zur Verwaltung der Paare in der Menge
 * P bei der Berechnung der Gröbner-Basis
 * @author Nils Przigoda
 * @version 0.1
 *
 */
class Paar {
	int[] Paar;
	boolean genutzt;
	
	Paar() {
		Paar = new int[2];
		genutzt = false;
	}
	
	Paar(int i, int j) {
		Paar = new int[2];
		Paar[0] = i;
		Paar[1] = j;
		genutzt = false;
	}	
}

/**
 * Eine Klasse, die Polynome mit beliebig vielen Veränderlichen
 * verarbeiten soll und dabei insbesondere den lt, lm, lc sowieso
 * multideg berechnen bzgl einer übergebenen Ordnung bestimmen können
 * soll.
 * @author Nils Przigoda
 * @version 0.1 
 */
public class Polynom {
	VektorBigInt[] poly;
	Rational[] koeff;
	
	/**
	 * Dier erzeugende-Aufruf, erstellt zunächst
	 * eines leeres Array für die die Koeffizienten
	 * und die Monome.
	 */
	public Polynom() {		
		poly = new VektorBigInt[0];
		koeff = new Rational[0];
	}
		
	/**
	 * Erzeuger für die Erstellung eines Polynoms mit
	 * einer vorgegebenen Liste von Koeffzienten und den
	 * dazugehörigen Monomen in Gestalt von Arrays
	 * @param p Das Monom-Array als VektorBigInt[]
	 * @param c Der KoeffizientenArray der Monome
	 */
	public Polynom(VektorBigInt[] p, Rational[] c) {		
		poly = p.clone();
		koeff = c.clone();				
	}
	
	/**
	 * Erzeuger für die Erstellung eines Polynoms mit
	 * nur einem Koeffzienten und dem dazugehörigen
	 * Monom
	 * @param p Das Monom als VektorBigInt
	 * @param c Der Koeffizient des Monoms
	 */
	public Polynom(VektorBigInt p, Rational c) {
		poly = new VektorBigInt[1];
		koeff = new Rational[1];
		poly[0] = p;
		koeff[0] = c;
	}

	/**
	 * multideg() ermittelt, wo das max. Monom liegt bzgl. <c, wobei
	 * @param c Der "Vergleichsvektor" bzw. die lineare 
	 * Kostenfunktion c
	 * @return Die Stelle an der sich das max. Monom befindet
	 */
	public int multideg(VektorBigInt c){
		int max_i = -1;
		BigInteger max = new BigInteger("0");		
		for (int i=0; i < poly.length; i++) {
			if (max.compareTo(poly[i].term_order_c(c)) < 0) {
				max = poly[i].term_order_c(c);
				max_i = i;
			}
		}		
		return max_i;	
	}
	
	/**
	 * Ermittelt das Leitmonom dieses Objekts
	 * @param c Der "Vergleichsvektor" bzw. die lineare 
	 * Kostenfunktion c
	 * @return Das Leitmonom wenn vorhanden
	 * @return null wenn keine Leitmonome vorhanden sind
	 */
	public VektorBigInt lm(VektorBigInt c){
		int max_i = multideg(c);
		if (max_i != -1 ) 
			return poly[max_i];
		else 			
			return null;		
	}
	
	/**
	 * Ermittel den Leitkoeffzienten dieses Objekts
	 * @param c Der "Vergleichsvektor" bzw. die lineare 
	 * Kostenfunktion c
	 * @return Den Leitkoeffzienten wenn vorhanden
	 * @return null wenn keine Leitkoeffzienten vorhanden sind
	 */
	public Rational lc(VektorBigInt c){
		int max_i = multideg(c);
		if (max_i != -1 ) 
			return koeff[max_i];
		else 			
			return null;		
	}
	
	/**
	 * Ermittel den Leitterm dieses Objekts
	 * @param c Der "Vergleichsvektor" bzw. die lineare 
	 * Kostenfunktion c
	 * @return Den Leitterm wenn vorhanden
	 * @return null wenn keine Terme vorhanden sind
	 */
	public Polynom lt(VektorBigInt c){
		int max_i = multideg(c);
		if (max_i != -1 ) 
			return new Polynom(poly[max_i], koeff[max_i]);
		else 
			return null;		
	}
	
	/**
	 * Ermittel das kgv vom lm von diesen einem übergebenen und 
	 * bzlg <c
	 * @param q Polynom mit dem das kgv berechnet werden soll 
	 * @param c Der "Vergleichsvektor" bzw. die lineare 
	 * Kostenfunktion c 
	 * @return
	 */
    public BigInteger[] kgV (Polynom q, VektorBigInt c) {
    	/*
    	 * Basteln einen Array der die Länge des erstens Monoms hat,
    	 * d.h. X1^0 X2^0 X3^1 hat die Länge 3, weil es 3 verschiedene 
    	 * Variablen gibt und fülle alle Einträge mit einer BigInt 0. 
    	 */    
    	BigInteger[] tmp = new BigInteger[this.poly[0].length];
    	Arrays.fill(tmp, new BigInteger("0"));    	
    	/*
    	 * Bestimme nun das Leitmonom von q:
    	 */
    	VektorBigInt q_lm = new VektorBigInt();
    	if (q.lm(c) != null)
    		q_lm = q.lm(c);
    	else {
    		o("null-Object in q.lm(c)");
    		w();
    		return null;
    	}
    	/*
    	 * Bestimme nun das Leitmonom dieses Objekts 
    	 * und speicher es in p_lm:
    	 */
    	VektorBigInt p_lm = new VektorBigInt();
    	if (this.lm(c) != null)
    		p_lm = this.lm(c);
    	else{
    		o("null-Object in this.lm(c)");
    		w();
    		return null;
    	}
    	
    	/*
    	 * Wenn nun beide nicht != 0 sind:
    	 * Trage an die Stellen jeweils das max von und 
    	 * p_lm und q_lm der entsprechenden Variable, z.B. X1,
    	 * ein: 
    	 */
    	for (int i=0; i< tmp.length; i++)
    		tmp[i] = p_lm.vektor[i].max(q_lm.vektor[i]);    	
		return tmp; 
    }
    
    /**
     * Ermittel das kgv von 2 BigIntegers mit Hilfe der
     * bereits vorhanden Funktion für gcd, ggT.
     * @param a Erster Wert
     * @param b Zweiter Wert
     * @return das kgv der beiden BigIntegers
     */
    public BigInteger kgV (BigInteger a, BigInteger b) {
    	return a.multiply( (b.divide( a.gcd(b) )) );
    }
    
    
    private static void o(Object o) {
		System.out.println(o);
	}
    
    private static void w() {
		try {
			o("wait for Input");
			System.in.read();			
		} catch (IOException e) {
			e.printStackTrace();
		}	
    }
    
    /**
     * Eine Methode zum Ausgeben der Polynome
     */
    public void print() {
    	o("Polynom BEGINN " );
    	for (int i=0; i < this.poly.length;i++) {
    		//o("i=" + i);
    		System.out.print(koeff[i].toString() + " "); 
    		poly[i].printZ();
    	} o("Polynom ENDE");
    	
    }       
     
    /**
     * Die Funktion kuerze streicht alle Monome mit mit 
     * Koeffizient 0.
     * @param polynom Das zu kürzende Polynom
     * @return null wenn alle Monome Koeff. 0 haben 
     * @return das gekürzte Polynom
     */
	private Polynom kuerze(Polynom polynom) {
		/*
		 * Zunächst einaml soll gezählt werden, wieviele
		 * Monom den Koeffzienten 0 haben:
		 */
		int raus = 0, length=polynom.koeff.length;
    	for (int i=0; i < polynom.koeff.length;i++) {
    		if (polynom.koeff[i].equals(new Rational(0,1)))
    			raus++;
    	}  
    	
    	/*
    	 * Wenn length - raus == 0 ist, so haben alle Monome,
    	 * den Koeff. 0 und wir können direkt die null-Referenz
    	 * für ein leeres Polynom zurückgeben
    	 */
    	if (length-raus == 0) 
    		return null;	
    	
    	/*
    	 *  Dann wird ein neues noch leeres Polynom erzeugt,
    	 *  in dem später die restlichen Monome mit Koeffizienten
    	 *  gespeichert werden sollen. 
    	 */    	
    	Polynom tmp = new Polynom();
    	
    	/*
    	 * jetzt werden die Längen der Arrays angepasst, und
    	 * zwar entsprechend der Länge des alten Polynoms-die 
    	 * Anzahl Monom mit Koeff. 0:
    	 */
    	tmp.poly = new VektorBigInt[length-raus];
    	tmp.koeff = new Rational[length-raus];
    	/* Setze raus wieder auf 0 um danach zu gucken,
    	 * und nutze ihn ab jetzt als counter zum speichern
    	 * der Monome die übernommen werden sollen:
    	 */
    	raus = 0; 
    	
    	/*
    	 * Jetzt durchlaufe die Liste des übergebenen Polynoms
    	 * und übernehme nur die Monom mit Koeff != 0:
    	 */ 
    	for (int i=0; i < length; i++) {
    		if (!polynom.koeff[i].equals(new Rational(0,1))) {    			
    			tmp.poly[raus] = polynom.poly[i];
    			tmp.koeff[raus] = polynom.koeff[i];
    			raus++;
    		}
    	}    	
    	/*
    	 * Ansonsten gibt jetzt unser tmp-Polynom mit den 
    	 * Monomen != 0 zurück:
    	 */
    	return tmp;
	}
   
	/**
	 * Add addiert zu diesem Polynom das übergeben Polynom q hinzu.
	 * @param q das zu addierende Polynom
	 * @return Die Summe beider Polynome
	 */    	
    public Polynom Add(Polynom q) {
    	/*
    	 * Ermitteln der Länge des neuen Polynoms über die
    	 * Laenge der Summe der beiden einzel Längen 
    	 */
    	int length = this.poly.length + q.poly.length, 
    		counter = this.poly.length;
    	/*
    	 * Erzeuge ein leeres Polynom, zu dem wir die summierten
    	 * Monom dann hinzufügen können:
    	 */
    	Polynom tmp = new Polynom();
    	/*
    	 * Jetzt noch die Laenge anpassen:
    	 */
    	tmp.poly = new VektorBigInt[length];
    	tmp.koeff = new Rational[length];
    	/*
    	 * Jetzt wird ein boolean-Array der Größe lenght von q
    	 * erstellt, dort werden zunächst alle Werte auf false
    	 * gesetzt, und im Folgenden dann auf true gesetzt, wenn
    	 * das Monom addiert wurde:
    	 */
    	boolean[] eingetragen = new boolean[q.poly.length];
    	Arrays.fill(eingetragen, false);
    	
    	// Läuft die Monome dieses Objekts ab:
    	for (int i=0;i < this.poly.length; i++) {
			// Läuft die Monome von q ab:
    		for (int j=0; j < q.poly.length; j++) {
    			
    			/* 
    			 * Zunächst einmal gucken ob die Monom, die
    			 *  gleichen Exponenten haben und ob das Monom 
    			 *  auch wirklich noch nicht addiert wurde:
    			 */
    			if (this.poly[i].equals(q.poly[j])
    					&& eingetragen[j] == false) {
    				/*
    				 * Wenn tmp noch keinen Eintrag an der 
    				 * i-ten Stelle hat, ausrechnen:
    				 */
    				if (tmp.poly[i] != null) {    					
    					tmp.poly[i] = tmp.poly[i];
    					tmp.koeff[i] = tmp.koeff[i].add(q.koeff[j]);
    					length--; // Die Länge muss auch um 1 reduziert werden
    				} else {
    					tmp.poly[i] = this.poly[i];
    					tmp.koeff[i] = this.koeff[i].add(q.koeff[j]);
    				}
    				/* 
    				 * Im bool-Array markieren, dass das Monom addiert 
    				 * wurde
    				 */
    				eingetragen[j]= true; 
    			}
    		}
    		/* q nun einmal druchlaufen ist und in tmp.poly[i] 
    		 * noch nichts steht, d.h. es wurde nichts zu tmp.poly[i]
    		 * addiert, müssten wir den Wert so an der i-ten Stelle eintragen:
    		 */
    		if (tmp.poly[i] == null) {
				tmp.poly[i] = this.poly[i];
				tmp.koeff[i] = this.koeff[i];
			}    		    	
    	}
    	
    	/*
    	 * Wenn nun alle Einträge von i-verglichen worden sind,
    	 * müssen wir noch im bool-Array nach nicht eingetragen/
    	 * addierten Monomen suchen: 
    	 */
    	for (int j=0; j < q.poly.length; j++) {
    		if (!eingetragen[j]) {
    			tmp.poly[counter] = q.poly[j];
				tmp.koeff[counter] = q.koeff[j];
				counter++;
    		}
    	}
    	/* Ne Zuweisung für das Ziel abhängig von der length
    	 * und zwar einfach den Bereich von von 0 bis counter 
    	 * via Array-Copy dem Ziel hinzufügen
    	 */
    	Polynom ziel = new Polynom();
    	ziel.poly = Arrays.copyOfRange(tmp.poly, 0, counter);
    	ziel.koeff = Arrays.copyOfRange(tmp.koeff, 0, counter);
    	    	
    	// Jetzt noch Kürzen, falls ein Koeff. o geworden ist:    	
    	return kuerze(ziel);
    }           

    
	/**
     * Sub zieht von diesem Polynom das übergebene Polynom q ab.
     * @param q Das Polynom das abgezogen werden soll

     * @return das Ergebnis der Subtraktion
     */
	public Polynom Sub(Polynom q) { 
		
		/*
    	 * Ermitteln der Länge des neuen Polynoms über die
    	 * Laenge der Summe der beiden einzel Längen 
    	 */
		int length = this.poly.length + q.poly.length, 
			counter = this.poly.length;
		/*
    	 * Erzeuge ein leeres Polynom, zu dem wir die summierten
    	 * Monom dann hinzufügen können:
    	 */	
    	Polynom tmp = new Polynom();
		/*
    	 * Jetzt noch die Laenge anpassen:
    	 */
    	tmp.poly = new VektorBigInt[length];
    	tmp.koeff = new Rational[length];
		/*
    	 * Jetzt wird ein boolean-Array der Größe lenght von q
    	 * erstellt, dort werden zunächst alle Werte auf false
    	 * gesetzt, und im Folgenden dann auf true gesetzt, wenn
    	 * das Monom addiert wurde:
    	 */
    	boolean[] eingetragen = new boolean[q.poly.length];
    	Arrays.fill(eingetragen, false);
    	
    	// Läuft die Monome dieses Objekts ab:
    	for (int i=0;i < this.poly.length; i++) {
			// Läuft die Monome von q ab:
    		for (int j=0; j < q.poly.length; j++) {
    			
    			/* 
    			 * Zunächst einmal gucken ob die Monom, die
    			 *  gleichen Exponenten haben und ob das Monom 
    			 *  auch wirklich noch nicht addiert wurde:
    			 */
    			if (this.poly[i].equals(q.poly[j])
    					&& eingetragen[j] == false) {
    				/*
					 * Wenn schon ein Eintrag vorhanden ist,
					 * berechne noch tmp koeff - q koeff:
					 */
    				if (tmp.poly[i] != null) {    					    					
    					tmp.koeff[i] = tmp.koeff[i].sub(q.koeff[j]);
    					length--; 	/* Die Länge muss auch um 1 reduziert werden,
    					 			 * weil ?
    					 			 */
    				} else {    					
    					/*
        				 * Wenn tmp noch keinen Eintrag an der 
        				 * i-ten Stelle hat, ausrechnen:
        				 */
    					tmp.poly[i] = this.poly[i];    					    					
    					tmp.koeff[i] = this.koeff[i].sub(q.koeff[j]);
    				}
    				/* 
    				 * Im bool-Array markieren, dass das Monom addiert 
    				 * wurde
    				 */
    				eingetragen[j]= true;
    			}
    		}
    		/* Nächstes Eintrag von this.poly[i+1] und wenn in tmp.poly[i] 
    		 * noch nichts steht, d.h. es wurde nichts zu tmp.poly[i]
    		 * addiert, müssen wir den Wert so an der i-ten Stelle eintragen:
    		 */
    		if (tmp.poly[i] == null) {
				tmp.poly[i] = this.poly[i];
				tmp.koeff[i] = this.koeff[i];
			}    		    	
    	}
    	
    	/*
    	 * Wenn nun alle Einträge von i-verglichen worden sind,
    	 * müssen wir noch im bool-Array nach nicht eingetragen/
    	 * addierten Monomen suchen: 
    	 */
    	for (int j=0; j < q.poly.length; j++) {
    		if (!eingetragen[j]) {
    			tmp.poly[counter] = q.poly[j];
				tmp.koeff[counter] = q.koeff[j].multiply(new Rational(-1,1));
				counter++;
    		}
    	}
    	/* Ne Zuweisung für das Ziel abhängig von der length
    	 * und zwar einfach den Bereich von von 0 bis counter 
    	 * via Array-Copy dem Ziel hinzufügen
    	 */
    	Polynom ziel = new Polynom();    	
    	ziel.poly = Arrays.copyOfRange(tmp.poly, 0, counter);    	
    	ziel.koeff = Arrays.copyOfRange(tmp.koeff, 0, counter);

    	// Jetzt noch Kürzen, falls ein Koeff. o geworden ist:
    	return kuerze(ziel);    
	}
	
	/**
	 * Mult multipliziert 2 Polynome miteinander this*q
	 * @param q mit diesem Polynom wird multipliziert
	 * @return das Ergebnis der Multiplikation
	 */
	public Polynom Mult(Polynom q) {
		/*
		 * Bestimme zunächst Produkt:
		 * k = Anzahl Monome this * Anzahl Monome q
		 * und setze c = 0 als counter für Einträge
		 */
		int k = this.koeff.length * q.koeff.length, 
			c = 0;
		/*
		 * Jetzt bestimme man Vektoren für die Monome und Koeff.
		 * die entsprechende Arrays sollten die Größe k haben:
		 */
		VektorBigInt[] pols = new VektorBigInt[k];
		Rational[] koeffs = new Rational[k];
		
		/*
		 * Jetzt einfach Stück für Stück ausrechnen: 
		 */
		for (int i=0; i < this.koeff.length; i++) {
			for (int j=0; j < q.koeff.length; j++) {
				pols[c] = this.poly[i].Add(q.poly[j]);
				koeffs[c] = this.koeff[i].multiply(q.koeff[j]);			
				c++;
			}			
		}

		return kuerze(new Polynom(pols,koeffs));
	}

    /**
     * Div teilt this durch das übergebenen Polynome, jedoch nur
     * wenn dies aus nicht mehr als einem Monom besteht 
     * @param q Divisor-Polynom mit max. einem Monom
     * @return this / q
     */
	public Polynom Div(Polynom q) {
		// Polynome mit mehr als einem Monom in q werden nicht ausgerechnet...
		if (q.koeff.length > 1) {
			o("Das ist mir noch zu hoch");
			return null;
		}
		// Wenn die Länge kleinergleich 0 ist, gibt es halt nichts zum teilen
		if (q.koeff.length <= 0) {
			o("Da gibt es nix zum teilen");
			return null;
		}		
		 		
		/* Jetzt eigentlich Division realisieren
    	 * Erzeuge ein leeres Polynom, zu dem wir die summierten
    	 * Monom dann hinzufügen können:
    	 */	
    	Polynom tmp = new Polynom();
		/*
    	 * Jetzt noch die Laenge anpassen:
    	 */
    	tmp.poly = new VektorBigInt[this.koeff.length];
    	tmp.koeff = new Rational[this.koeff.length];
    			
		for (int i=0; i < this.koeff.length ; i++) {			
			tmp.poly[i] = this.poly[i].Sub(q.poly[0]);			
			tmp.koeff[i] = this.koeff[i].divide(q.koeff[0]);
		}

		return kuerze(tmp);
	}	

    /**
     * Eine Funktion zum Testen, ob alle Koeffizient gleich 0 sind.
     * @return true wenn Koeffiziente 0 sind.
     * @return false sonst, mind. 1 Koeffizient ungleich 0
     */
	public boolean zero() {		
		// TODO Auto-generated method stub
		for (int i=0; i < koeff.length; i++)
			if (!koeff[i].equals(new Rational(0,1)))
				return false;
		return true;
	}
	
	/**
	 * Bestimmt das S-Polynom: 
	 * S(p,q) =   kgV( lm(p), lm (q) ) / lt(p) * p 
	 * 			- kgV( lm(p), lm (q) ) / lt(q) * q
	 * @param p Erstes Polynom
	 * @param q Zweites Polynom
	 * @param M1c Wert zur Bestimmung der Monomordnung
	 * @return das S-Polynom von p und q
	 */
	public static Polynom Spol(Polynom p, Polynom q, VektorBigInt M1c) {
		/*
		 * Wir speichern zunächst einmal das kgV( lm(p), lm (q) )
		 * im BigInteger-Array kgv_lm, was für kgV der Leitmonome stehen
		 * soll.
		 */
		BigInteger[] kgv_lm = p.kgV(q, M1c);						

		/*
		 * Jetzt wird das kgV in den Typ Polynom mit Koeff. 1 
		 * umgewandelt, weil die Objekten mit Polynomen leichter
		 * rechnen können. 
		 */
		Polynom kgv_lm_p = new Polynom(
				new VektorBigInt(kgv_lm), new Rational(1,1));
		
		/*
		 * Bestimmung des Leitterms von p und q 
		 * und speichern als lt_p bzw lt_q, welche Objekte
		 * vom Typ Polynom sind: 
		 */
		Polynom lt_p = p.lt(M1c);
		Polynom lt_q = q.lt(M1c);
		
		/*
		 * Als nächstes werden jetzt kgv/lt ausgerechnet und 
		 * in tmp1 der Teil bzgl p. und in tmp2 bzgl. q ge-
		 * speichert: 
		 */
		Polynom tmp_1 = kgv_lm_p.Div(lt_p);
		Polynom tmp_2 = kgv_lm_p.Div(lt_q);
		
		/*
		 * Danach werden die tmp_1/2 mit p bzw q multipliziert: 
		 */
		tmp_1 = tmp_1.Mult(p);
		tmp_2 = tmp_2.Mult(q); 
			
		/*
		 * Zu guter letzt jetzt die Subtraktion 
		 */
		return tmp_1.Sub(tmp_2);
	}
	
	/**
	 * GB bestimmt mit Hilfe des Buchberger-Algorithmus die
	 * Gröbner Basen zur übergebenen Basis, bzw. dem Erzeugern
	 * @param erzeuger Die Basis respektive die Erzeuger
	 * @param M1c Die Kostenfunktion zur Monomordnung
	 * @return die Gröbner-Basis des Erzeugers/der Basis
	 */
	public static Polynom[] GB(Polynom[] erzeuger, VektorBigInt M1c) {
		/*
		 * Erzeuge eine Menge Basis, in der alle Erzeuger enthalten sind,
		 * sowie eine Kopie dieser Menge für andere Bearbeitungszwecke.
		 */
		Polynom[] Basis =  erzeuger.clone();		
		Polynom[] Basis_tmp = erzeuger.clone();
		/*
		 * Dann erzeuge einen Array mit den Paaren aus der Basis
		 * das sind Basis.length^2-Basis.length, außerdem setze 
		 * lauf = Basis.length
		 */
		int lauf = Basis.length,
			counter = lauf * lauf - lauf;
		Paar[] Paare = new Paar[counter];		
		counter = 0;
		for (int i=0; i < lauf; i++) {
			for (int j=0; j < lauf; j++) {
				if (i != j) {
					Paare[counter] = new Paar(i,j);					
					counter++;
				}
			}
		}
		Paar[] Paare_tmp = Paare.clone();
		
		/* Erzeuge nen Bool P_leer um zu merken ob die Paar-Menge
		 * erzeugt wurde:
		 */		
		boolean P_leer = true;
		
		/*
		 * Die while-Schleife des Buchberger-Alg.
		 */
		do {			
			P_leer = true;
			/* 
			 * Für chose {f,g} \in P nehmen wir einfach das erste
			 * Element, was noch nicht gestrichen wurde.
			 */
			
			for (int i=0; i < Paare.length; i++) {
				if (!Paare[i].genutzt) {
					P_leer = false;
					Paare[i].genutzt = true;
					/*
					 * Berechne zunächst das S-Polynom für Einträgen
					 * des Paares und dies sind die Indezes des Basis-Arrays:
					 */
					Polynom c = Spol(Basis[Paare[i].Paar[0]], 
									 Basis[Paare[i].Paar[1]],
									 M1c);
					c = Reduce(Basis,c, M1c);
					if (c != null) {
						// Setze B := B u {c}
						Basis_tmp = new Polynom[Basis.length+1]; 
						for (int k=0; k < Basis.length; k++) {
							Basis_tmp[k] = Basis[k];
						} Basis_tmp[Basis.length] = c;
						Basis = new Polynom[Basis_tmp.length];
						Basis = Basis_tmp.clone();
						
						/* P = P u {{b,c} | b\ in B }
						 * Füge nun zu P die neuen Paare {b,c}
						 * mit b \in B hinzu
						 */
						Paare_tmp = new Paar[Paare.length + Basis.length-1]; 
						for (int k=0; k < Paare.length; k++) {
							Paare_tmp[k] = Paare[k];
						}
						for (int k=Paare.length; k < Paare_tmp.length; k++) {
							Paare_tmp[k] = new Paar(k-Paare.length,Basis.length-1);
						}						
						Paare = new Paar[Paare_tmp.length];
						Paare = Paare_tmp.clone();											
					}
				}
			}
			
		} while(!P_leer); // solange P nicht leer ist.
		
		return Basis;
	}
	
	public static Polynom[] redGB(Polynom[] basis, VektorBigInt M1c) {
		
		boolean red = false;
				
		do {			
			red = false;
			for (int i=0; i < basis.length; i++) {
				Polynom b = Polynom.StronglyReduce(ohne(basis,i),
					basis[i], M1c);
				if (b == null) {
					basis = ohne(basis,i);
					red=true;
				} else {
					if (!b.equals(basis[i])) {
						basis[i] = b;
						red=true;
					} 					
				}
			}
		} while(red);
		for (int i=0; i < basis.length; i++) {
			basis[i] = basis[i].Norm(M1c);
		}
		return basis;
	}
	
	private Polynom Norm(VektorBigInt M1c) {		
		Rational lc = (this.lc(M1c));		
		lc = lc.invert();		
		Polynom tmp = new Polynom();
		tmp.koeff = this.koeff.clone();
		tmp.poly = this.poly.clone();
		for (int i=0; i < tmp.koeff.length;i++) {
			tmp.koeff[i] = tmp.koeff[i].multiply(lc);
		}
		return tmp;
	}
	
	public int Gib_Var_Anz(){
		if (this.poly.length > 0)
			return this.poly[0].length;
		else
			return -1;
	}
	
	public VektorBigInt KanBij() {
		if (this.koeff.length > 2  ) {
			o("Fehler");
			return null;
		}
		VektorBigInt tmp = new VektorBigInt(this.poly[0].length);
		for (int i=0; i < this.poly.length;i++) {
			if ( this.koeff[i].equals(new Rational(1,1))) {
				for (int j=0; j < this.poly[i].length;j++) {
					if (! this.poly[i].vektor[j].equals(new BigInteger("0"))) {
						tmp.vektor[j] = this.poly[i].vektor[j];
					}
				}						
			}
			if ( this.koeff[i].equals(new Rational(-1,1))) {
				for (int j=0; j < this.poly[i].length;j++) {
					if (! this.poly[i].vektor[j].equals(new BigInteger("0"))) {
						tmp.vektor[j] = this.poly[i].vektor[j].
						multiply(new BigInteger("-1"));
					}
				}						
			}
				
		}
		return tmp;
	}
	
	public VektorBigInt[] Gib_Pol(){
		return this.poly;
	}
	
	public Rational[] Gib_Koeff(){
		return this.koeff;
	}

	public static Polynom Reduce(Polynom[] basis, Polynom f, VektorBigInt M1c) {
		Polynom[] J = new Polynom[basis.length];
		// Erstellen der Menge J:= { b \in B || lm(b)|lm(f) }		
		VektorBigInt lmf = f.lm(M1c);
		int count = 0;
		for (int i=0; i < basis.length; i++) {
			VektorBigInt tmp = basis[i].lm(M1c);
			if (lmf != null)
				if (Teiler(tmp,lmf)) {
					J[count] = basis[i];
					count++;
				}
		}
		if (count == 0){ // J ist leere Menge: 			
			return f;			
		}
		J = Arrays.copyOfRange(J, 0, count);
		
		Polynom tmp = f.lt(M1c);
		/* Liefert eine int-Pseudo-Zufallszahl im Bereich von 0 bis count-1.
		 * Dies soll die choose b \in J Zeile des Pseudo-Codes repräsentieren:
		 */
		int zufall_J = (int) (count * Math.random());
		/*
		 * Berechne nun f - ( lt(f) / lt(b) ) b
		 */	
		tmp = tmp.Div( J[zufall_J].lt(M1c) );
		tmp = tmp.Mult(J[zufall_J]);		
		f = f.Sub(tmp);
		if (f == null)
			return f;
		return Reduce(basis, f, M1c);		
	}	
	
	public static Polynom StronglyReduce(Polynom[] basis, Polynom f, VektorBigInt M1c) {
		J_Paar[] J = new J_Paar[basis.length*f.koeff.length];
		/* Erstellen der Menge J:= { (b,m) \in B x Mf | lm(b)|m) }
		 * wobei m \in Mf ein beliebiges Monom von f sei.
		 * Die Menge kann max. basis.length*f.koeff.length Elemente enthalten
		 */				
		int count = 0;
		for (int i=0; i < f.poly.length; i++) {
			VektorBigInt m = f.poly[i];
			for (int j=0; j < basis.length; j++) {
				VektorBigInt lm_b = basis[j].lm(M1c);
				if (lm_b != null && m != null) {
					if (Teiler(lm_b, m)) {
						J[count] = new J_Paar( basis[j],
								new Polynom(f.poly[i], f.koeff[i]));
						count++;
					}
				}
			}
		}
		if (count == 0){ // J ist leere Menge: 			
			return f;
		}
		J = Arrays.copyOfRange(J, 0, count);
				
		/* Liefert eine int-Pseudo-Zufallszahl im Bereich von 0 bis count-1.
		 * Dies soll die choose b \in J Zeile des Pseudo-Codes repräsentieren:
		 */
		int zufall_J = (int) (count * Math.random());
		/*
		 * Berechne nun f - ( f_m / lt(b) ) * b
		 */
		Polynom f_m = J[zufall_J].monom;
		Polynom lt_b = J[zufall_J].poly.lt(M1c);
		Polynom tmp = f_m.Div(lt_b);
		tmp = tmp.Mult(J[zufall_J].poly);
		f = f.Sub(tmp);
		if (f == null)
			return f;
		return StronglyReduce(basis, f, M1c);		
	}
	
	/**
	 * 
	 * @return
	 */
	public static Polynom ohne() {
		return null;
	}
	
	/**
	 * Teiler ermittelt von tmp lmf teilt. lm(b) | lm(f)
	 * @param lmb 
	 * @param lmf
	 * @return
	 */
	private static boolean Teiler(VektorBigInt lmb, VektorBigInt lmf) {
		for (int i=0; i < lmb.length;i++) {			
			if (lmb.vektor[i].compareTo(lmf.vektor[i]) > 0) 
				return false;
		}
		return true;
	}
	
	private static Polynom[] ohne (Polynom[] basis, int ohne_Nr) {
		Polynom[] ziel = new Polynom[basis.length-1];
		Polynom[] tmp = Arrays.copyOfRange(basis, 0, ohne_Nr);
		for (int i=0; i < ohne_Nr; i++) {
			ziel[i] = tmp[i];
		}
		tmp = Arrays.copyOfRange(basis, 
				ohne_Nr+1, basis.length);
		for (int i=ohne_Nr; i < ziel.length; i++) {
			ziel[i] = tmp[i-ohne_Nr];
		}		
		return ziel;
	}	

}