package AlZAGK.numbers.Rational;

import java.math.BigInteger;
import java.util.StringTokenizer;

/**
 * @author Anton Mellit
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class Rational implements Comparable{
    private BigInteger num;
    private BigInteger denom;
    public static final Rational ZERO = new Rational(0, 1);
    public static final Rational ONE = new Rational(1, 1);
    
    /**
     * creates rational 0/1
     */
    public Rational() {
    	this.num = new BigInteger("0");
    	this.denom = new BigInteger("1");
    }
    
    public Rational(long num, long denom) {
        this.num = BigInteger.valueOf(num);
        this.denom = BigInteger.valueOf(denom);
        normalize();
    }
    
    public Rational(BigInteger num, BigInteger denom) {
        this.num = num;
        this.denom = denom;
    	normalize();
    }
    
    public long getNum() {
        return num.longValue();
    }

    public long getDenom() {
        return denom.longValue();
    }
			
    protected void normalize() {
    	if(denom.compareTo(BigInteger.ZERO)<0) {
    		num = num.negate();
    		denom = denom.negate();
    	} else if(denom.equals(BigInteger.ZERO)) {
            throw new Error("Zero in denominator");
    	}
    	BigInteger g = denom.gcd(num);
    	num = num.divide(g);
    	denom = denom.divide(g);
    }
    
    public Rational add(Rational r) {
        return new Rational(num.multiply(r.denom).
        	add(denom.multiply(r.num)),
        	denom.multiply(r.denom));
    }    

    public Rational add(Rational r, long k) {
        return new Rational(num.multiply(r.denom).
        	add(denom.multiply(r.num).multiply(BigInteger.valueOf(k))),
        	denom.multiply(r.denom));
    }
    
    public Rational sub(Rational r) {
        return this.add(r.multiply(new Rational(-1,1)));        
    }

    public Rational multiply(Rational r) {
        return new Rational(num.multiply(r.num), denom.multiply(r.denom));
    }

    public Rational divide(Rational r) {
        return new Rational(num.multiply(r.denom), denom.multiply(r.num));
    }

    public Rational multiply(long k) {
        return new Rational(num.multiply(BigInteger.valueOf(k)), denom);
    }

    public Rational divide(long k) {
        return new Rational(num, denom.multiply(BigInteger.valueOf(k)));
    }

    public String toString() {
        if(denom.equals(BigInteger.ONE)) {
            return String.valueOf(num);
        } else {
            return num + "/" + denom;
        }
    }
    
    public boolean isZero() {
        return num.equals(BigInteger.ZERO);
    }

    public boolean equals(Object obj) {
        if (obj instanceof Rational) {
            Rational rational = (Rational) obj;
            return (rational.num.equals(num)) && (rational.denom.equals(denom));
        } else return false;
    }

    public int compareTo(Object o) {
        Rational rational = (Rational) o;
        return num.multiply(rational.denom).compareTo(denom.multiply(rational.num));
    }

    public Rational getGCD(Rational r) {
        return new Rational(num.multiply(r.denom).gcd(denom.multiply(r.num)), denom.multiply(r.denom));
    }

    public Rational getAbs() {
        return new Rational(num.abs(), denom);
    }

    public Rational invert() {
        return new Rational(denom, num);
    }

    public static Rational valueOf(long x) {
        return new Rational(x, 1);
    }
    
    public static Rational parse(String str) {
    	StringTokenizer tok = new StringTokenizer(str, "-/", true);
   		boolean minus = false;
   		String num = "0";
   		String denom = "1";
   		boolean slash = false;
   		while(tok.hasMoreTokens()) {
   			String t;
   			do {
   				t = tok.nextToken().trim();
   			} while(tok.hasMoreTokens() && t.length()==0);
   			if(t.equals("-")) {
   				minus = true;
   			} else if(t.equals("/")) {
   				slash = true;
   			} else if(t.length()>0) {
   				if(!slash) {
   					num = t;
   				} else {
   					denom = t;
   				}
   			}
   		}
   		long numVal = Long.parseLong(num);
   		long denomVal = Long.parseLong(denom);
   		if(minus) {
   			numVal = -numVal;
   		}
   		return new Rational(numVal, denomVal);
    }

    public boolean isInteger() {
        return denom.equals(BigInteger.ONE);
    }
    
    public int hashCode() {
        return (int)((num.hashCode()<<16)^denom.hashCode());
    }
    
    protected static long mod(long a, long b) {
        long c = a%b;
        if(c<0) {
            c = c + Math.abs(b);
        }
        return c;
    }

    protected static long gcd(long a, long b) {
        while(b!=0) {
            long c = mod(a, b);
            a = b;
            b = c;
        }
        return Math.abs(a);
    }
}
